@@ -1,6 +1,6 @@
 ############################################################
 #
-#   $Id: Linux.pm 128 2010-05-12 13:16:44Z trevor $
+#   $Id$
 #   Sys::Filesystem - Retrieve list of filesystems and their properties
 #
 #   Copyright 2004,2005,2006 Nicola Worthington
@@ -23,15 +23,18 @@ package Sys::Filesystem::Linux;
 
 # vim:ts=4:sw=4:tw=78
 
+use 5.008001;
+
 use strict;
 use warnings;
 use vars qw($VERSION @ISA);
 
 use Carp qw(croak);
-require IO::File;
-require Sys::Filesystem::Unix;
+use Cwd 'abs_path';
+use IO::File              ();
+use Sys::Filesystem::Unix ();
 
-$VERSION = '1.28';
+$VERSION = '1.406';
 @ISA     = qw(Sys::Filesystem::Unix);
 
 sub version()
@@ -40,22 +43,23 @@ sub version()
 }
 
 # Default fstab and mtab layout
-my @keys = qw(fs_spec fs_file fs_vfstype fs_mntops fs_freq fs_passno);
+my @keys       = qw(fs_spec fs_file fs_vfstype fs_mntops fs_freq fs_passno);
 my %special_fs = (
-                   swap                    => 1,
-                   proc                    => 1,
-                   devpts                  => 1,
-                   tmpfs                   => 1,
-                   sysfs                   => 1,
-                   securityfs              => 1,
-                   debugfs                 => 1,
-                   procbususb              => 1,
-                   udev                    => 1,
-                   nfsd                    => 1,
-                   binfmt_misc             => 1,
-                   fusectl                 => 1,
-                   'fuse.gvfs-fuse-daemon' => 1,
-                 );
+    binfmt_misc             => 1,
+    debugfs                 => 1,
+    devpts                  => 1,
+    fusectl                 => 1,
+    'fuse.gvfs-fuse-daemon' => 1,
+    mini_fo                 => 1,
+    nfsd                    => 1,
+    proc                    => 1,
+    procbususb              => 1,
+    securityfs              => 1,
+    swap                    => 1,
+    sysfs                   => 1,
+    tmpfs                   => 1,
+    udev                    => 1,
+);
 
 sub new
 {
@@ -65,8 +69,9 @@ sub new
 
     # Defaults
     $args{fstab} ||= '/etc/fstab';
-    $args{mtab} ||= -r '/proc/self/mounts' ? '/proc/self/mounts' : '/etc/mtab';
+    $args{mtab} ||= -r '/proc/mounts' ? '/proc/mounts' : '/etc/mtab';
     #$args{xtab}  ||= '/etc/lib/nfs/xtab';
+    $args{canondev} and $self->{canondev} = 1;
 
     local $/ = "\n";
 
@@ -77,18 +82,15 @@ sub new
         {
             next if ( /^\s*#/ || /^\s*$/ );
             my @vals = split( ' ', $_ );
-            if ( $vals[0] =~ /^\s*LABEL=(.+)\s*$/ )
-            {
-                $self->{ $vals[1] }->{label} = $1;
-            }
+            $vals[0] =~ /^\s*LABEL=(.+)\s*$/
+              and $self->{ $vals[1] }->{label} = $1;
+            $args{canondev} and -l $vals[0] and $vals[0] = abs_path( $vals[0] );
             $self->{ $vals[1] }->{mount_point} = $vals[1];
             $self->{ $vals[1] }->{device}      = $vals[0];
             $self->{ $vals[1] }->{unmounted}   = 1;
-            $self->{ $vals[1] }->{special}     = 1 if ( defined( $special_fs{ $vals[2] } ) );
-            for ( my $i = 0; $i < @keys; ++$i )
-            {
-                $self->{ $vals[1] }->{ $keys[$i] } = $vals[$i];
-            }
+            defined $special_fs{ $vals[2] }
+              and $self->{ $vals[1] }->{special} = 1;
+            @{ $self->{ $vals[1] } }{@keys} = @vals;
         }
         $fstab->close();
     }
@@ -103,6 +105,8 @@ sub new
         croak "Unable to open fstab file ($args{mtab})\n";
     }
 
+    delete $self->{canondev};
+
     $self;
 }
 
@@ -143,25 +147,25 @@ be queried as methods through the parent L<Sys::Filesystem> object.
 
 =item fs_spec
 
-Dscribes the block special device or remote filesystem to be mounted.
+Describes the block special device or remote filesystem to be mounted.
 
 For  ordinary  mounts  it  will hold (a link to) a block special device
-node (as created by L<mknod(8)>)  for  the  device  to  be  mounted,  like
-/dev/cdrom’   or   ‘/dev/sdb7’.    For   NFS   mounts  one  will  have
-<host>:<dir>, e.g., ‘knuth.aeb.nl:/’.  For procfs, use ‘proc’.
+node (as created by L<mknod(8)>) for  the  device  to  be  mounted, like
+'/dev/cdrom' or '/dev/sdb7'.  For NFS mounts one will  have <host>:<dir>,
+e.g., 'knuth.aeb.nl:/'.  For procfs, use 'proc'.
 
 Instead of giving the device explicitly, one may indicate the (ext2  or
 xfs)  filesystem that is to be mounted by its UUID or volume label (cf.
 L<e2label(8)> or  L<xfs_admin(8)>),  writing  LABEL=<label>  or  UUID=<uuid>,
-e.g.,   ‘LABEL=Boot’   or  ‘UUID=3e6be9de-8139-11d1-9106-a43f08d823a6’.
+e.g., 'LABEL=Boot' or 'UUID=3e6be9de-8139-11d1-9106-a43f08d823a6'.
 This will make the system more robust: adding or removing a  SCSI  disk
 changes the disk device name but not the filesystem volume label.
 
 =item fs_file
 
 Describes the mount point for the filesystem. For swap partitions,
-this field should be specified as‘none. If the name of the mount
-point contains spaces these can be escaped as‘\040.
+this field should be specified as 'none'. If the name of the mount
+point contains spaces these can be escaped as '\040'.
 
 =item fs_vfstype
 
@@ -185,10 +189,10 @@ least  the type of mount plus any additional options appropriate to the
 filesystem type.  For documentation on the available options  for  non-
 nfs  file systems, see L<mount(8)>.  For documentation on all nfs-specific
 options have a look at L<nfs(5)>.  Common for all types of file system are
-the options ‘‘noauto’’ (do not mount when 'mount -a' is given, e.g., at
-boot time), ‘‘user’’ (allow a user  to  mount),  and  ‘‘owner’’  (allow
-device  owner to mount), and ‘‘_netdev’’ (device requires network to be
-available).  The ‘‘owner’’ and ‘‘_netdev’’ options are  Linux-specific.
+the options 'noauto' (do not mount when 'mount -a' is given, e.g., at
+boot time), 'user' (allow a user  to  mount),  and  'owner'  (allow
+device  owner to mount), and '_netdev' (device requires network to be
+available).  The 'owner' and '_netdev' options are  Linux-specific.
 For more details, see L<mount(8)>.
 
 =item fs_freq
@@ -218,7 +222,7 @@ L<Sys::Filesystem>, L<Sys::Filesystem::Unix>, L<fstab(5)>
 
 =head1 VERSION
 
-$Id: Linux.pm 128 2010-05-12 13:16:44Z trevor $
+$Id$
 
 =head1 AUTHOR
 
@@ -230,7 +234,7 @@ Jens Rehsack <rehsack@cpan.org> - L<http://www.rehsack.de/>
 
 Copyright 2004,2005,2006 Nicola Worthington.
 
-Copyright 2009,2010 Jens Rehsack.
+Copyright 2009-2014 Jens Rehsack.
 
 This software is licensed under The Apache Software License, Version 2.0.
 
